#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include "nksmf.h"
#include "nksmf-lib.h"

int smfOpenFile(SMFFILE *smf, char *filename) {
  smf->fp = fopen(filename, "rb");
  if (!smf->fp) {
    return errno;
  }
  
  smf->firsttrack_pos = SMF_HEADERSIZE;
  smf->firsttrack = NULL;
  
  return 0;
}

int smfCloseFile(SMFFILE *smf) {
  if ( !fclose(smf->fp) ) {
    return errno;
  }
  
  return 0;
}

int smfHasValidHeader(SMFFILE *smf) {
  char buf[SMF_VALIDHEADERLEN];
  size_t ret;
  int i;
  
  if (!smf || !smf->fp) {
    return 0;
  }
  
  rewind(smf->fp);
  ret = fread(buf, sizeof(char), SMF_VALIDHEADERLEN, smf->fp);
  
  if ( ret < SMF_VALIDHEADERLEN || memcmp(buf, SMF_VALIDHEADERSTR, SMF_VALIDHEADERLEN) != 0 ) {
    return 0; /* failed */
  }
  
  return 1; /* success */
}

int smfGetHeaderChunk(SMFFILE *smf, SMFHEADER *header) {
  size_t ret;

  rewind(smf->fp);
  ret = fread(header, SMF_HEADERSIZE, 1, smf->fp);
  
  if (ret < 1) {
    if ( feof(smf->fp) ) {
      return EOF;
    } else {
      return errno;
    }
  }

  header->length = convendian_uint32(header->length);
  header->format = convendian_uint16(header->format);
  header->numtrack = convendian_uint16(header->numtrack);
  header->timebase = convendian_uint16(header->timebase);
  
  return 0;
}

/* GfBAϊ֐ (4oCg̕p) */
long unsigned int convendian_uint32(long unsigned int obj) {
  long unsigned int ret = 0;
  int i;
  
  for (i=0; i<4; i++) {
    ret = ret << 8;
    ret |= ((unsigned char *)&obj)[i];
  }
  
  return ret;
}

/* GfBAϊ֐ (2oCg̕p) */
short unsigned int convendian_uint16(short unsigned int obj) {
  short unsigned int ret = 0;
  int i;
  
  for (i=0; i<2; i++) {
    ret = ret << 8;
    ret |= ((unsigned char *)&obj)[i];
  }
  
  return ret;
}
