#include <stdio.h>
#include <windows.h>
#include <mmsystem.h>

int main(void) {
  HMIDIOUT hMidiOut;
  MIDIOUTCAPS outCaps;
  MMRESULT res;
  DWORD vol, orig_vol;

  res = midiOutGetDevCaps(MIDI_MAPPER, &outCaps, sizeof(outCaps));
  if (res != MMSYSERR_NOERROR) {
    printf("midiOutGetDevCaps() failed.\n");
    return 1;
  }
  if (outCaps.dwSupport & MIDICAPS_VOLUME) {
    printf("Volume changes are supported.\n");
  } else {
    printf("Volume changes are NOT supported.\n");
    return 1;
  }
  if (outCaps.dwSupport & MIDICAPS_LRVOLUME) {
    printf("Separate volume changes are supported.\n");
  }

  res = midiOutOpen(&hMidiOut, MIDI_MAPPER, (DWORD_PTR)NULL, 0, CALLBACK_NULL);
  if (res != MMSYSERR_NOERROR) {
    printf("Cannot open MIDI output device.\n");
    return 1;
  }

  midiOutGetVolume(hMidiOut, &vol);
  orig_vol = vol;
  if (outCaps.dwSupport & MIDICAPS_LRVOLUME) {
    printf("Left channel volume: %X\n", (vol & 0x0000FFFF));
    printf("Right channel volume: %X\n", ((vol & 0xFFFF0000) >> 16));
  } else {
    printf("Mono volume level: %X\n", (vol & 0x0000FFFF));
  }

  midiOutShortMsg(hMidiOut, 0x00FF3c90);
  Sleep(1000);
  midiOutShortMsg(hMidiOut, 0x00003c90);

  Sleep(500);
  midiOutSetVolume(hMidiOut, 0x60006000);
  Sleep(500);

  midiOutGetVolume(hMidiOut, &vol);
  if (outCaps.dwSupport & MIDICAPS_LRVOLUME) {
    printf("Left channel volume: %X\n", (vol & 0x0000FFFF));
    printf("Right channel volume: %X\n", ((vol & 0xFFFF0000) >> 16));
  } else {
    printf("Mono volume level: %X\n", (vol & 0x0000FFFF));
  }

  midiOutShortMsg(hMidiOut, 0x00FF3c90);
  Sleep(1000);
  midiOutShortMsg(hMidiOut, 0x00003c90);

  midiOutSetVolume(hMidiOut, orig_vol);

  midiOutReset(hMidiOut);
  midiOutClose(hMidiOut);

  return 0;
}
