#include <stdio.h>
#include <windows.h>
#include <mmsystem.h>

void show_all_input_devices(void);
void CALLBACK MidiInProc(HMIDIIN hMidiIn, UINT wMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2);

int endprog_flag;
MIDIHDR header;

#define BUFSIZE 256

int main(int argc, char **argv) {
  HMIDIIN hMidiIn;
  MMRESULT res;
  UINT devid;
  char errmsg[MAXERRORLENGTH];

  show_all_input_devices();

  if (argc > 1) {
    sscanf(argv[1], "%u", &devid);
  } else {
    devid = 0u;
  }

  ZeroMemory(&header, sizeof(MIDIHDR));
  header.lpData = (char*)malloc(BUFSIZE);
  header.dwBufferLength = BUFSIZE;
  header.dwFlags = 0;

  printf("Opening MIDI input device (ID=%u)...\n", devid);
  res = midiInOpen(&hMidiIn, devid, (DWORD_PTR)MidiInProc, 0, CALLBACK_FUNCTION);
  if (res != MMSYSERR_NOERROR) {
    printf("Cannot open MIDI input device: ");
    midiInGetErrorText(res, errmsg, sizeof(errmsg));
    printf("%s\n", errmsg);
    return 1;
  }

  midiInPrepareHeader(hMidiIn, &header, sizeof(MIDIHDR));
  midiInAddBuffer(hMidiIn, &header, sizeof(MIDIHDR));

  midiInStart(hMidiIn);

  endprog_flag = 0;
  while ( !endprog_flag ) {
    Sleep(10);
  }

  midiInStop(hMidiIn);
  midiInReset(hMidiIn);

  midiInUnprepareHeader(hMidiIn, &header, sizeof(MIDIHDR));

  printf("Closing MIDI input device...\n");
  midiInClose(hMidiIn);

  free(header.lpData);

  return 0;
}

void show_all_input_devices(void) {
  MIDIINCAPS inCaps;
  MMRESULT res;
  UINT num, devid;

  num = midiInGetNumDevs();
  printf("Number of input devices: %d\n", num);
  for (devid=0; devid<num; devid++) {
    res = midiInGetDevCaps(devid, &inCaps, sizeof(inCaps));
    if (res != MMSYSERR_NOERROR) { continue; }
    printf("ID=%d: %s\n", devid, inCaps.szPname);
  }

  printf("\n");
}

void CALLBACK MidiInProc(HMIDIIN hMidiIn, UINT wMsg, DWORD dwInstance, DWORD dwParam1, DWORD dwParam2) {
  MIDIHDR *hdr;
  int i;

  switch (wMsg) {
    case MIM_OPEN:
      printf("Open\n");
      break;
    case MIM_CLOSE:
      printf("Close\n");
      break;
    case MIM_DATA:
      if ( (dwParam1 & 0x000000f0) == 0x90 &&
           (dwParam1 & 0x0000ff00) >> 8 == 0x15u && (dwParam1 & 0x00ff0000) != 0 ) {
        endprog_flag = 1;
        printf("endprog_flag is on.\n");
      }
      if ( (dwParam1 & 0x000000ff) != 0xfe ) {
        printf("MIM_DATA: wMsg=%08X, p1=%08X, p2=%08X\n", wMsg, dwParam1, dwParam2);
      }
      break;
    case MIM_LONGDATA:
      printf("MIM_LONGDATA ");
      hdr = (MIDIHDR*)dwParam1;
      printf("[%d bytes]: ", hdr->dwBytesRecorded);
      for (i=0; i<hdr->dwBytesRecorded; i++) {
        printf("%02X ", (unsigned char)((hdr->lpData)[i]));
      }
      printf("\n");
      break;
    case MIM_ERROR:
    case MIM_LONGERROR:
    case MIM_MOREDATA:
    default:
      printf("Others: wMsg=%08X, p1=%08X, p2=%08X\n", wMsg, dwParam1, dwParam2);
      break;
  }
}
