#include <stdio.h>
#include <stdlib.h>
#include <windows.h>
#include <mmsystem.h>

#define BUFSIZE 256

HMIDIOUT hMidiOut;
int end_of_program;

void CALLBACK MidiInProc(HMIDIIN hMidiIn, UINT wMsg, DWORD dwInstance,
                          DWORD dwParam1, DWORD dwParam2) {
  MIDIHDR *hdr, outhdr;
  int i;

  switch (wMsg) {
    case MIM_OPEN:
      printf("MIDI device is opened.\n");
      break;
    case MIM_CLOSE:
      printf("MIDI device is closed.\n");
      break;
    case MIM_DATA:
      midiOutShortMsg(hMidiOut, dwParam1);
      
      if ( (dwParam1 & 0x0000fff0) == 0x00003C80 ||
           (dwParam1 & 0x00fffff0) == 0x00003C90    ) {
        end_of_program = 1;
        printf("end_of_program flag is on.\n");
      }

      if ( (dwParam1 & 0x000000ff) != 0xfe ) {
        printf("MIM_DATA: wMsg=%08X, p1=%08X, p2=%08X\n",
               wMsg, dwParam1, dwParam2);
      }
      
      break;
    case MIM_LONGDATA:
      printf("MIM_LONGDATA: ");
      hdr = (MIDIHDR*)dwParam1;
      printf("[%d bytes] ", hdr->dwBytesRecorded);
      for (i=0; i<hdr->dwBytesRecorded; i++) {
        printf("%02X ", (unsigned char)((hdr->lpData)[i]));
      }
      printf("\n");
      
      ZeroMemory(&outhdr, sizeof(MIDIHDR));
      outhdr.lpData = (LPSTR)(hdr->lpData);
      outhdr.dwBufferLength = hdr->dwBytesRecorded;
      outhdr.dwFlags = 0;
      midiOutPrepareHeader(hMidiOut, &outhdr, sizeof(MIDIHDR));
      midiOutLongMsg(hMidiOut, &outhdr, sizeof(MIDIHDR));

      midiInPrepareHeader(hMidiIn, hdr, sizeof(MIDIHDR));
      midiInAddBuffer(hMidiIn, hdr, sizeof(MIDIHDR));

      midiOutUnprepareHeader(hMidiOut, &outhdr, sizeof(MIDIHDR));

      break;
    case MIM_ERROR:
    case MIM_LONGERROR:
    case MIM_MOREDATA:
    default:
      printf("MidiInProc: wMsg=%08X, p1=%08X, p2=%08X\n",
             wMsg, dwParam1, dwParam2);
      break;
  }
}

int main(int argc, char **argv) {
  HMIDIIN hMidiIn;
  MMRESULT res;
  MIDIHDR header;
  UINT devid;
  char errmsg[MAXERRORLENGTH];

  if (argc > 1) {
    sscanf(argv[1], "%u", &devid);
  } else {
    devid = 0u;
  }

  res = midiOutOpen(&hMidiOut, MIDI_MAPPER, (DWORD_PTR)NULL, 0, CALLBACK_NULL);
  if (res != MMSYSERR_NOERROR) {
    printf("Cannot open MIDI output device.\n");
    return 1;
  }
  printf("Succeeded to open a MIDI output device.\n");

  ZeroMemory(&header, sizeof(MIDIHDR));
  header.lpData = (char*)malloc(BUFSIZE);
  header.dwBufferLength = BUFSIZE;
  header.dwFlags = 0;

  res = midiInOpen(&hMidiIn, devid, (DWORD_PTR)MidiInProc, 0, CALLBACK_FUNCTION);
  if (res != MMSYSERR_NOERROR) {
    printf("Cannot open MIDI input device (ID=0x%X): ", devid);
    midiInGetErrorText(res, errmsg, sizeof(errmsg));
    printf("%s\n", errmsg);
    return 1;
  }
  printf("Succeeded to open a MIDI input device (ID=%u).\n", devid);

  midiInPrepareHeader(hMidiIn, &header, sizeof(MIDIHDR));
  midiInAddBuffer(hMidiIn, &header, sizeof(MIDIHDR));

  midiInStart(hMidiIn);

  end_of_program = 0;
  while ( !end_of_program ) {
    Sleep(10);
  }

  midiInStop(hMidiIn);
  midiInReset(hMidiIn);

  midiInUnprepareHeader(hMidiIn, &header, sizeof(MIDIHDR));

  midiInClose(hMidiIn);
  printf("Closed a MIDI input device.\n");

  free(header.lpData);

  midiOutReset(hMidiOut);
  midiOutClose(hMidiOut);
  printf("Closed a MIDI output device.\n");

  return 0;
}
